﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Messaging;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace WebServerDocxGeneration
{
    class WebServerDocxGeneration
    {
        static string ReturnQueueLocalName = @".\PRIVATE$\GenerateDocumentsClient";

        static MessageQueue ReturnQueue;

#if true // true if running the daemon on the same machine as the process calling the daemon
        public static FileInfo TemplateDocumentFileInfo = new FileInfo(@"C:\Users\Public\Documents\TemplateDocuments\TemplateDocument.docx");
        static DirectoryInfo GeneratedDocumentsDirectory = new DirectoryInfo(@"C:\Users\Public\Documents\GeneratedDocuments\");
        static string GenerateDocumentsQueueRemoteName = @".\PRIVATE$\GenerateDocuments";
        static string ReturnQueueRemoteName = @".\PRIVATE$\GenerateDocumentsClient";
#else
        public static FileInfo TemplateDocumentFileInfo = new FileInfo(@"\\MINI-1\Mini-1-C\Users\Public\Documents\TemplateDocuments\TemplateDocument.docx");
        static DirectoryInfo GeneratedDocumentsDirectory = new DirectoryInfo(@"\\MINI-1\Mini-1-C\Users\Public\Documents\GeneratedDocuments\");
        static string GenerateDocumentsQueueRemoteName = @"FormatName:Direct=OS:mini-1\PRIVATE$\GenerateDocuments";
        static string ReturnQueueRemoteName = @"FormatName:Direct=OS:iwin\PRIVATE$\GenerateDocumentsClient";
#endif

        static void Main(string[] args)
        {
            Initialize();
            GenerateDocuments();
        }

        /*
         * Outgoing message:
         * 
         * Message label: GenerateDocuments
         * 
         * <GenerateDocuments>
         *   <ReturnQueue Name=".\PRIVATE$\GenerateDocumentClient" />
         *   <TemplateDocument Name="C:\Users\Public\Documents\TemplateDocuments\TemplateDocument.docx" />
         *   <GeneratedDocumentsDirectory Name="C:\Users\Public\Documents\GeneratedDocuments" />
         *   <DocumentGenerationData>
         *     <!-- the XML here is specific to the template document.  If the template document changes, the schema of the following will change to match it. -->
         *     <Customers>
         *       <Customer>
         *         <CustomerID>1</CustomerID>
         *         <Name>Cheryl</Name>
         *         <HighValueCustomer>True</HighValueCustomer>
         *         <Orders>
         *           <Order>
         *             <ProductDescription>Unicycle</ProductDescription>
         *             <Quantity>3</Quantity>
         *             <OrderDate>9/5/2001</OrderDate>
         *           </Order>
         *           <Order>
         *             <ProductDescription>Tricycle</ProductDescription>
         *             <Quantity>3</Quantity>
         *             <OrderDate>8/6/2000</OrderDate>
         *           </Order>
         *         </Orders>
         *       </Customer>
         *     </Customers>
         *   </DocumentGenerationData>
         * </GenerateDocuments>
         */

        /*
         * Return message:
         * 
         * Message label: DocumentGenerationComplete
         * 
         * <DocumentsGenerated>
         *   <Document Name="48FCE74D-E9E1-49DB-ABCD-629196B6AEE5.docx" />
         * </DocumentsGenerated>
         */
        private static void GenerateDocuments()
        {
            MessageQueue GenerateDocumentsQueue = new MessageQueue(GenerateDocumentsQueueRemoteName);
            Message generateDocumentsMessage = new Message();

            for (int i = 0; i < 1000; ++i)
            {
                var gmsg = new XElement("GenerateDocuments",
                    new XElement("ReturnQueue", new XAttribute("Name", ReturnQueueRemoteName)),
                    new XElement("TemplateDocument", new XAttribute("Name", TemplateDocumentFileInfo.FullName)),
                    new XElement("GeneratedDocumentsDirectory", new XAttribute("Name", GeneratedDocumentsDirectory.FullName)),
                    new XElement("DocumentGenerationData",
                        new XElement("Customers",
                            new XElement("Customer",
                                new XElement("CustomerID", 1),
                                new XElement("Name", "Cheryl"),
                                new XElement("HighValueCustomer", true),
                                new XElement("Orders",
                                    new XElement("Order",
                                        new XElement("ProductDescription", "Unicycle"),
                                        new XElement("Quantity", 3),
                                        new XElement("OrderDate", "9/5/2001")),
                                    new XElement("Order",
                                        new XElement("ProductDescription", "Tricycle"),
                                        new XElement("Quantity", 3),
                                        new XElement("OrderDate", "9/5/2001")))))));

                generateDocumentsMessage.Body = gmsg.ToString(SaveOptions.DisableFormatting);
                generateDocumentsMessage.Label = "GenerateDocuments";
                GenerateDocumentsQueue.Send(generateDocumentsMessage);

                Message returnMessage = new Message();

                returnMessage = ReturnQueue.Receive();
                returnMessage.Formatter = new XmlMessageFormatter(new String[] { "System.String,mscorlib" });
                var returnMessageText = returnMessage.Body.ToString();

                if (returnMessage.Label == "DocumentGenerationComplete")
                {
                    var generateDocumentsArguments = XElement.Parse(returnMessageText);

                    // If you want to validate the XML in the message, this is the place to do it.

                    foreach (var generatedDocument in generateDocumentsArguments.Elements("Document"))
                    {
                        var generatedDocumentfileName = Path.Combine(GeneratedDocumentsDirectory.FullName, (string)generatedDocument.Attribute("Name"));
                        Console.WriteLine("Generated document: {0}", generatedDocumentfileName);
                        FileInfo fi = new FileInfo(generatedDocumentfileName);
                        fi.Delete();
                    }
                }
                else
                    Console.WriteLine("Unexpected return message");
                System.Threading.Thread.Sleep(100);
            }
        }

        private static void Initialize()
        {
            if (!GeneratedDocumentsDirectory.Exists)
                GeneratedDocumentsDirectory.Create();

            if (MessageQueue.Exists(ReturnQueueLocalName))
                MessageQueue.Delete(ReturnQueueLocalName);
            ReturnQueue = MessageQueue.Create(ReturnQueueLocalName, false);
        }
    }
}
